/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 2002 Dmitriy Nikitinskiy
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include <CORBA-SMALL.h>
#include <mico/template_impl.h>

#include "foxmico.h"

#ifdef WIN32
#include <winsock2.h>
#endif

FXDEFMAP(FOXProxy) FOXProxyMap[]={
  FXMAPFUNCS(SEL_IO_READ,0,MAXKEY,FOXProxy::onFileEvents),
  FXMAPFUNCS(SEL_IO_WRITE,0,MAXKEY,FOXProxy::onFileEvents),
  FXMAPFUNCS(SEL_IO_EXCEPT,0,MAXKEY,FOXProxy::onFileEvents),
  FXMAPFUNCS(SEL_TIMEOUT,0,MAXKEY,FOXProxy::onTimerEvents),
};

FXIMPLEMENT(FOXProxy,FXObject,FOXProxyMap,ARRAYNUMBER(FOXProxyMap))

FOXProxy::FOXProxy(FOXDispatcher* d) : disp(d) {};

long FOXProxy::onFileEvents(FXObject*, FXSelector sel, void* handle){
  FXInputHandle i=(FXInputHandle)handle;
  printf("onFileEvent %d\n",i);
  return disp->input_callback(i);
};
long FOXProxy::onTimerEvents(FXObject*, FXSelector sel, void*){
  return disp->timer_callback(sel & 0xffff);
};

long
FOXDispatcher::input_callback (FXInputHandle sel)
{
  list<FileEvent*>::iterator i(fevents.begin());
  while(i!=fevents.end() && (*i)->fid!=sel) 
    i++;
  if(i!=fevents.end()){
    printf("EventFound %d\n",sel);
    FileEvent *event = *i;
    event->cb->callback(this, event->ev);
#ifdef WIN32
    u_long data;
    do{
        ioctlsocket((*i)->fd,FIONREAD,&data);
        if(data)
          event->cb->callback(this, event->ev);           
    } while(data);
    if(!WSAResetEvent(sel)){
      printf("WSAResetEvent error!\n");
    };
#endif   
    return 1;
  }
  else
    return 0;
}

long
FOXDispatcher::timer_callback (FXSelector sel)
{
  list<TimerEvent*>::iterator i(tevents.begin());
  while(i!=tevents.end() && (*i)->sel!=sel) 
    i++;
  if(i!=tevents.end()){

    TimerEvent *event = *i;

    tevents.erase (i);
    
    event->cb->callback(this, Timer);
    delete event;
    return 1;    
  }
  else
    return 0;
}

FOXDispatcher::FOXDispatcher (FXApp* a)
    : app(a), tsel(0), fxproxy(this)
{
}

FOXDispatcher::~FOXDispatcher ()
{
    list<FileEvent*>::iterator i;
    for (i = fevents.begin(); i != fevents.end(); ++i) {
	(*i)->cb->callback (this, Remove);
    app->removeInput((*i)->fid,(*i)->fxmode);
    delete *i;
    };	

    list<TimerEvent*>::iterator j;
    for (j = tevents.begin(); j != tevents.end(); ++j){
	(*j)->cb->callback (this, Remove);
	app->removeTimeout((*j)->tid);
        delete *j;
    };	
}

void
FOXDispatcher::rd_event (CORBA::DispatcherCallback *cb, CORBA::Long fd)
{
#ifdef WIN32
    FXInputHandle i=WSACreateEvent();
    if(WSAEventSelect(fd,i,FD_READ|FD_ACCEPT)){
        printf("rd_event Error\n");
    };
    printf("rd_event fd: %d Event: %d\n",fd,i);
#else
    FXInputHandle i=fd;
#endif
    FileEvent *ev= new FileEvent(i,cb,Read,INPUT_READ);
    ev->fd=fd;
    app->addInput(i,INPUT_READ,&fxproxy,0);
    fevents.push_back (ev);
}

void
FOXDispatcher::wr_event (CORBA::DispatcherCallback *cb, CORBA::Long fd)
{
#ifdef WIN32
    FXInputHandle i=WSACreateEvent();
    WSAEventSelect(fd,i,FD_WRITE);
#else
    FXInputHandle i=fd;
#endif
    FileEvent *ev= new FileEvent(i,cb,Write,INPUT_WRITE);
    app->addInput(i,INPUT_WRITE,&fxproxy,0);
    fevents.push_back (ev);

}

void
FOXDispatcher::ex_event (CORBA::DispatcherCallback *cb, CORBA::Long fd)
{
#ifdef WIN32
    FXInputHandle i=WSACreateEvent();
    WSAEventSelect(fd,i,FD_OOB);
#else
    FXInputHandle i=fd;
#endif
    FileEvent *ev= new FileEvent(i,cb,Except,INPUT_EXCEPT);
    app->addInput(i,INPUT_EXCEPT,&fxproxy,0);
    fevents.push_back (ev);
}

void
FOXDispatcher::tm_event (CORBA::DispatcherCallback *cb, CORBA::ULong tmout)
{
    TimerEvent *ev= new TimerEvent(app->addTimeout(tmout,&fxproxy,tsel),cb,tsel);
    tsel++;
    tevents.push_back (ev);
}

void
FOXDispatcher::remove (CORBA::DispatcherCallback *cb, Event e)
{
    if (e == All || e == Timer) {
	list<TimerEvent*>::iterator i, next;
	for (i = tevents.begin(); i != tevents.end(); i = next) {
	    next = i;
	    ++next;
	    if ((*i)->cb == cb) {
		app->removeTimeout((*i)->tid);
		delete *i;
		tevents.erase (i);
	    }
	}
    }
    if (e == All || e == Read || e == Write || e == Except) {
	list<FileEvent*>::iterator i, next;
	for (i = fevents.begin(); i != fevents.end(); i = next) {
	    next = i;
	    ++next;
	    if ((*i)->cb == cb && (e == All || e == (*i)->ev)) {
                app->removeInput((*i)->fid,(*i)->fxmode);
                delete *i;
		fevents.erase (i);
	    }
	}
    }
}

void
FOXDispatcher::run (CORBA::Boolean infinite)
{
    do {
	   app->runOneEvent();
    } while (infinite);
}

void
FOXDispatcher::move (CORBA::Dispatcher *)
{
    assert (0);
}

CORBA::Boolean
FOXDispatcher::idle () const
{
    return fevents.size() + tevents.size() == 0;
}
