// -*- c++ -*-
/*
 *  MICO --- an Open Source CORBA implementation
 *  Copyright (c) 1997-2001 by The Mico Team
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the MICO Home Page at
 *  http://www.mico.org/
 */

#ifndef __mico_FOX_h__
#define __mico_FOX_h__

#include "fx.h"

class FOXDispatcher;

class FOXProxy : public FXObject {
  FXDECLARE(FOXProxy)
protected:
  FOXDispatcher* disp;
  FOXProxy() {};
public:
  FOXProxy(FOXDispatcher* d);
  
  long onFileEvents(FXObject*, FXSelector, void*);
  long onTimerEvents(FXObject*, FXSelector, void*);
};

class FOXDispatcher :  public CORBA::Dispatcher {
    FXApp* app;
    FXint tsel;
    FOXProxy fxproxy;
   
    struct FileEvent {
	FXInputHandle fid;
	CORBA::DispatcherCallback *cb;
	Event ev;
	FXint fxmode;
//Win2000 Bug >>>
    FXint fd;
//Win2000 Bug <<<

	FileEvent (FXInputHandle _fid, CORBA::DispatcherCallback *_cb, Event _ev, FXint _fxmode)
	    : fid (_fid), cb (_cb), ev (_ev), fxmode(_fxmode)
	{  };
     private:
	FileEvent () {}
    };
    struct TimerEvent {
	FXTimer* tid;
	CORBA::DispatcherCallback *cb;
	FXSelector sel;

	TimerEvent (FXTimer* _tid, CORBA::DispatcherCallback *_cb, FXSelector s)
	    : tid (_tid), cb (_cb), sel(s)
	{  }
      private:
	TimerEvent () {}
    };
    list<FileEvent*> fevents;
    list<TimerEvent*> tevents;

public:

    FOXDispatcher (FXApp*);
    virtual ~FOXDispatcher ();

    virtual void rd_event (CORBA::DispatcherCallback *, CORBA::Long fd);
    virtual void wr_event (CORBA::DispatcherCallback *, CORBA::Long fd);
    virtual void ex_event (CORBA::DispatcherCallback *, CORBA::Long fd);
    virtual void tm_event (CORBA::DispatcherCallback *, CORBA::ULong tmout);
    virtual void remove (CORBA::DispatcherCallback *, Event);
    virtual void run (CORBA::Boolean infinite = TRUE);
    virtual void move (CORBA::Dispatcher *);
    virtual CORBA::Boolean idle () const;

    long input_callback (FXInputHandle);
    long timer_callback (FXSelector);

};

#endif // __mico_FOX_h__
